@@ -1,9 +1,11 @@
+# $Id: Filter.pm 2357 2008-06-20 17:41:54Z rcaputo $
+
 package POE::Filter;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '1.294_512'; # NOTE - Should be #.### (three decimal places)
+$VERSION = do {my($r)=(q$Revision: 2357 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 use Carp qw(croak);
 
@@ -164,16 +166,14 @@ The L</SYNOPSIS> shows get_one_start() in use.
 
 get_one() parses zero or one complete item from the filter's internal
 buffer.  The data is returned as an ARRAYREF suitable for passing to
-another filter or a POE::Wheel object.  Filters will return empty
-ARRAYREFs if they don't have enough raw data to build a complete item.
+another filter or a POE::Wheel object.
 
 get_one() is the lazy form of get().  It only parses only one item at
 a time from the filter's buffer.  This is vital for applications that
 may switch filters in mid-stream, as it ensures that the right filter
 is in use at any given time.
 
-The L</SYNOPSIS> shows get_one() in use.  Note how it assumes the
-return is always an ARRAYREF, and it implicitly handles empty ones.
+The L</SYNOPSIS> shows get_one() in use.
 
 =head2 get ARRAYREF
 
@@ -183,9 +183,6 @@ filter's internal buffer.  It then parses as many full items as
 possible from the buffer and returns them in another array reference.
 Any unprocessed data remains in the filter's buffer for the next call.
 
-As with get_one(), get() will return an empty array reference if the
-filter doesn't contain enough raw data to build a complete item.
-
 In fact, get() is implemented in POE::Filter in terms of
 get_one_start() and get_one().
 
@@ -216,7 +213,7 @@ Here's the get() form of the SYNOPSIS stand-alone example:
 get() should not be used with wheels that support filter switching.
 Its greedy nature means that it often parses streams well in advance
 of a wheel's events.  By the time an application changes the wheel's
-filter, it's too late: The old filter has already parsed the rest of
+filter, it's too late: The old fiter has already parsed the rest of
 the received data.
 
 Consider a stream of letters, numbers, and periods.  The periods
@@ -362,4 +359,3 @@ Please see L<POE> for more information about authors and contributors.
 =cut
 
 # rocco // vim: ts=2 sw=2 expandtab
-# TODO - Edit.
